/***************************************************************************
     qgsgeorefdatapoint.h
     --------------------------------------
    Date                 : Sun Sep 16 12:02:56 AKDT 2007
    Copyright            : (C) 2007 by Gary E. Sherman
    Email                : sherman at mrcc dot com
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* $Id: qgsgeorefdatapoint.h 13436 2010-05-07 13:32:19Z mmassing $ */

#ifndef QGSGEOREFDATAPOINT_H
#define QGSGEOREFDATAPOINT_H

#include "qgsmapcanvasitem.h"

#include "qgscoordinatetransform.h"

class QgsGCPCanvasItem;
class QgsGeorefPluginGui;

class QgsGeorefDataPoint : public QObject
{
    Q_OBJECT

  public:
    //! constructor
    QgsGeorefDataPoint( QgsMapCanvas *srcCanvas, QgsMapCanvas *dstCanvas,
                        QgsGeorefPluginGui *pluginGui,
                        const QgsPoint& pixelCoords, const QgsPoint& mapCoords,
                        const QgsPoint& mapCoordsQgisCanvas, bool enable );
    QgsGeorefDataPoint( const QgsGeorefDataPoint &p );
    ~QgsGeorefDataPoint();

    //! returns coordinates of the point
    QgsPoint pixelCoords() const { return mPixelCoords; }
    void setPixelCoords( const QgsPoint &p );

    QgsPoint mapCoords() const { return mMapCoords; };
    void setMapCoords( const QgsPoint &p );

    QgsPoint mapCoordsQgisCanvas() const { return mMapCoordsQgisCanvas; };

    QgsPoint mapCoordsPluginCanvas() const;

    QgsGeorefPluginGui *pluginGui() const { return mPluginGui; };

    QgsCoordinateTransform * createTransformQgisPlugin( bool isToPlugin );
    void setTransform( QgsCoordinateTransform *coordTransform, bool isMapQgis );

    bool isEnabled() const { return mEnabled; };
    void setEnabled( bool enabled );

    int id() const { return mId; }
    void setId( int id );

    bool contains( const QPoint &p, bool isMapPlugin );

    QgsMapCanvas *srcCanvas() const { return mSrcCanvas; }
    QgsMapCanvas *dstCanvas() const { return mDstCanvas; }

    QPointF residual() const { return mResidual; }
    void setResidual( const QPointF& r );

  public slots:
    void moveTo( const QPoint &, bool isMapPlugin );
    void updateCoords();

  private:

    QgsMapCanvas *mSrcCanvas;
    QgsMapCanvas *mDstCanvas;

    QgsGCPCanvasItem *mGCPSourceItem;
    QgsGCPCanvasItem *mGCPDestinationItem;
    QgsPoint mPixelCoords;
    QgsPoint mMapCoords;
    QgsPoint mMapCoordsQgisCanvas;

    QgsGeorefPluginGui *mPluginGui;

    int mId;
    bool mEnabled;
    QPointF mResidual;
};

#endif //QGSGEOREFDATAPOINT_H
