/***************************************************************************
     qgsgeorefvalidators.cpp
     --------------------------------------
    Date                 : 14-Feb-2010
    Copyright            : (C) 2010 by Jack R, Maxim Dubinin (GIS-Lab)
    Email                : sim@gis-lab.info
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* $Id: qgsgeorefvalidators.cpp 13187 2010-03-28 22:14:44Z jef $ */

#include <QStringList>

#include "qgsgeorefvalidators.h"

QgsDMSAndDDValidator::QgsDMSAndDDValidator( QObject *parent )
    : QValidator( parent )
{
  mIsGeographic = false;
  mIsLongitude = false;
}

QValidator::State QgsDMSAndDDValidator::validate( QString &input, int &pos ) const
{
  Q_UNUSED( pos );

  // DDD.DDD
  QRegExp rx("^[-]?\\d*([.]\\d*)?$");
  if ( rx.exactMatch( input ) )
  {
    if ( ! mIsGeographic )
    {
      return Acceptable;
    }
    else
    {
      double maxCoord = mIsLongitude ? 180 : 90;
      double coord = input.toDouble();
      return ( coord > (-1 * maxCoord) && coord < maxCoord ) ? Acceptable : Invalid;
    }
  }
  if ( ! mIsGeographic )
  {
    return Invalid;
  }
  // DD MM SS.SSSS
  return validateDDmmSS(input);
}

QValidator::State QgsDMSAndDDValidator::validateDDmmSS( QString &input ) const
{
  // Regular Expressions for DD MM SS.sss
  QString re = "(^[-]?\\d{1,3}\\s{1}$)|"
               "(^[-]?\\d{1,3}\\s{1}\\d{1,2}\\s?$)|"
               "(^[-]?\\d{1,3}\\s{1}\\d{1,2}\\s{1}\\d{1,2}($|[.]{1}\\d*$))";

  QRegExp rx(re);
  if ( ! rx.exactMatch( input ) )
  {
    return Invalid;
  }

  // DD MM SS
  QStringList lst = input.split(" ", QString::SkipEmptyParts);
  int id = lst.size() - 1;
  double maxCoord = ( id == 0 ) ? ( mIsLongitude ? 180 : 90 ) : 60;
  double coord = lst.at(id).toDouble();

  return ( coord > (-1 * maxCoord) && coord < maxCoord ) ? Acceptable : Invalid;
}

void QgsDMSAndDDValidator::setTypeCoordinate(bool isGeographic, bool isLongitude)
{
  mIsGeographic = isGeographic;
  mIsLongitude = isLongitude;
}
